import Vue from "vue";
import VueRouter from "vue-router";
import { formatRoutes } from "@/utils/routerUtil";
import { i18n } from "@/utils/i18n";
import store from "@/store";
import { mergeI18nFromRoutes } from "@/utils/i18n";
import Layouts from "@/layouts/Layouts";

Vue.use(VueRouter);
//解决重复路由报错
const originalPush = VueRouter.prototype.push;
// 修改原型对象中的push方法
VueRouter.prototype.push = function push(location) {
  return originalPush.call(this, location).catch((err) => err);
};

const routes = [
  {
    path: "/login",
    name: "跳转页面",
    component: () => import("@/pages/login/Login"),
  },
  {
    path: "/jump",
    name: "跳转页面",
    component: () => import("@/pages/basicset/jump/jump"),
  },
  {
    path: "*",
    name: "404",
    component: () => import("@/pages/exception/404"),
  },
  {
    path: "/403",
    name: "403",
    component: () => import("@/pages/exception/403"),
  },
  {
    path: "/",
    name: "首页",
    component: Layouts,
    redirect: "/website",
    children: [
      {
        path: "/website",
        name: "站点管理",
        meta: {
          icon: "bank",
        },
        component: () => import("@/pages/basicset/site/website"),
      },
      {
        path: "/business",
        meta: {
          icon: "book",
        },
        component: () => import("@/pages/basicset/business/Index"),
        children: [
          {
            path: "",
            name: "业务事项管理",
            meta: {
              icon: "book",
              invisible: true,
            },
            component: () => import("@/pages/basicset/business/business"),
            redirect: "/business/businessmanage",
            children: [
              // 业务管理
              {
                path: "businessmanage",
                meta: {
                  invisible: true,
                },
                component: () =>
                  import("@/pages/basicset/business/components/businessTabs1"),
              },
              // 事项管理
              {
                path: "mattermanage",
                meta: {
                  invisible: true,
                },
                component: () =>
                  import("@/pages/basicset/business/components/businessTabs2"),
              },
              // 业务事项关联
              {
                path: "businessinmanage",
                meta: {
                  invisible: true,
                },
                component: () =>
                  import("@/pages/basicset/business/components/businessTabs3"),
              },
              // 办事指南事项管理
              {
                path: "workguide",
                meta: {
                  invisible: true,
                },
                component: () =>
                  import("@/pages/basicset/business/components/businessTabs4"),
              },
            ],
          },
          {
            path: "addmatter",
            name: "新增事项",
            component: () => import("@/pages/basicset/addmatter/AddMatter"),
            meta: { invisible: true },
          },
          {
            path: "synopsis",
            name: "事项概要",
            component: () => import("@/pages/basicset/addmatter/Synopsis"),
            meta: { invisible: true },
          },
        ],
      },
      {
        path: "/department",
        meta: {
          icon: "audit",
        },
        component: () => import("@/pages/basicset/dept/Index"),
        children: [
          {
            path: "",
            name: "部门及窗口管理",
            component: () => import("@/pages/basicset/dept/department"),
            meta: { invisible: true, keepAlive: true },
          },
          {
            path: "adddepartment",
            name: "新增窗口事项",
            meta: {
              invisible: true,
            },
            component: () => import("@/pages/basicset/dept/addWindowMatter"),
          },
        ],
      },
      {
        path: "/personnel",
        name: "工作人员管理",
        meta: {
          icon: "idcard",
        },
        component: () => import("@/pages/basicset/workman/personnel"),
      },
      {
        path: "/festival",
        name: "节假日管理",
        meta: {
          icon: "carry-out",
        },
        component: () => import("@/pages/basicset/holiday/festival"),
      },
      {
        path: "/deploy",
        name: "部署模块管理",
        meta: {
          icon: "appstore",
          roles: ["admin"],
        },
        component: () => import("@/pages/basicset/deploy/deploy"),
      },

      {
        path: "/hall",
        name: "大厅管理",
        meta: {
          icon: "gateway",
        },
        component: () => import("@/pages/basicset/hall/Hall"),
        redirect: "/hall/hallmanage",
        children: [
          {
            path: "hallmanage",
            name: "",
            component: () =>
              import("@/pages/basicset/hall/hallmanage/HallManage.vue"),
            meta: {
              invisible: true,
            },
          },
          {
            path: "hallwindow",
            name: "大厅窗口管理",
            meta: {
              invisible: true,
            },
            component: () =>
              import("@/pages/basicset/hall/hallwindow/HallWindow.vue"),
          },
        ],
      },

      {
        path: "/surface",
        component: () => import("@/pages/basicset/surface/index"),
        meta: {
          icon: "skin",
        },
        children: [
          {
            path: "",
            name: "皮肤管理",
            component: () => import("@/pages/basicset/surface/surface"),
            meta: { invisible: true },
          },
          {
            path: "addsurfacetemplate",
            name: "新增皮肤模板",
            component: () =>
              import("@/pages/basicset/surface/AddSurfaceTemplate"),
            meta: { invisible: true },
          },
        ],
      },
      {
        path: "/configurat",
        name: "短信配置",
        meta: {
          icon: "mail",
        },
        component: () => import("@/pages/basicset/sms/configurat"),
      },

      {
        path: "/appmarket",
        component: () => import("@/pages/basicset/appmarket/Index"),
        meta: {
          icon: "shop",
        },
        children: [
          {
            path: "",
            name: "应用集市",
            component: () => import("@/pages/basicset/appmarket/AppMarket"),
            meta: { invisible: true },
            redirect: "/appmarket/terminalapp",
            children: [
              {
                path: "terminalapp",
                component: () =>
                  import("@/pages/basicset/appmarket/components/TerminalApp"),
                meta: {
                  invisible: true,
                  // keepAlive: true,
                },
              },
              {
                path: "moveapp",
                component: () =>
                  import("@/pages/basicset/appmarket/components/MoveApp"),
                meta: {
                  invisible: true,
                  // keepAlive: true,
                },
              },
              {
                path: "blackapp",
                component: () =>
                  import("@/pages/basicset/appmarket/components/BlackApp"),
                meta: {
                  invisible: true,
                },
              },
              {
                path: "appTheme",
                component: () =>
                  import("@/pages/basicset/appmarket/components/AppTheme"),
                meta: {
                  invisible: true,
                },
              },
            ],
          },
          {
            path: "appdetails",
            name: "应用详情",
            component: () => import("@/pages/basicset/appmarket/AppDetails"),
            meta: { invisible: true },
            redirect: "/appmarket/appdetails/appdetailspage",
            children: [
              {
                path: "appdetailspage",
                component: () =>
                  import(
                    "@/pages/basicset/appmarket/components/AppDetailsPage"
                  ),
                meta: {
                  invisible: true,
                },
              },
              {
                path: "dataupdate",
                component: () =>
                  import("@/pages/basicset/appmarket/components/DataUpdate"),
                meta: {
                  invisible: true,
                },
              },
              {
                path: "fieldconfig",
                component: () =>
                  import("@/pages/basicset/appmarket/components/FieldConfig"),
                meta: {
                  invisible: true,
                },
              },
            ],
          },
        ],
      },
      {
        path: "/system",
        name: "系统设置",
        component: () => import("@/pages/basicset/system/System"),
        meta: {
          icon: "global",
        },
        redirect: "system/parameter",
        children: [
          {
            path: "user",
            name: "用户管理",
            component: () => import("@/pages/basicset/system/user/User"),
            meta: { invisible: true },
          },
          {
            path: "role",
            name: "角色管理",
            component: () => import("@/pages/basicset/system/role/Role"),
            meta: { invisible: true },
          },
          {
            path: "resource",
            name: "资源管理",
            component: () =>
              import("@/pages/basicset/system/resourceManage/Resource"),
            meta: { invisible: true },
          },
          {
            path: "dimension",
            name: "维度管理",
            component: () =>
              import("@/pages/basicset/system/dimension/Dimension"),
            meta: { invisible: true },
          },
          {
            path: "parameter",
            name: "系统参数",
            component: () =>
              import("@/pages/basicset/system/parameter/Parameter"),
            meta: { invisible: true },
          },
          {
            path: "task",
            name: "任务信息",
            component: () => import("@/pages/basicset/system/task/TaskSet"),
            meta: { invisible: true },
          },
          {
            path: "systemlogs",
            name: "操作日志",
            component: () =>
              import("@/pages/basicset/system/systemlogs/SystemLogs"),
            meta: { invisible: true },
          },
        ],
      },
    ],
  },
];
formatRoutes(routes);
const router = new VueRouter({
  routes,
});

// 不需要登录拦截的路由配置
const loginIgnore = {
  names: ["404", "403"], //根据路由名称匹配
  paths: ["/login", "/jump"], //根据路由fullPath匹配
  /**
   * 判断路由是否包含在该配置中
   * @param route vue-router 的 route 对象
   * @returns {boolean}
   */
  includes(route) {
    return this.names.includes(route.name) || this.paths.includes(route.path);
  },
};

/**
 * 初始化路由实例
 * @returns {VueRouter}
 */
function initRouter() {
  // 是否异步路由模式
  const options = store.state.setting.asyncRoutes
    ? require("./async/config.async").default
    : require("./config").default;
  formatRoutes(options.routes);
  return options;
}

// 判断权限
function permission(router, role) {
  if (router.meta && router.meta.roles) {
    return router.meta.roles.includes(role);
  } else {
    return true;
  }
}
// 计算动态路由
function calcRouters(dynamicRouter, role) {
  // 过滤
  let res = dynamicRouter.filter((v) => {
    // 判断是否有权限访问此路由
    if (permission(v, role)) {
      // 判断有没有子路由
      if (v.children) {
        // 递归
        v.children = calcRouters(v.children, role);
      }
      return true;
    } else {
      return false;
    }
  });
  return res;
}

// 动态菜单
function createMenus() {
  let role = store.getters["site/userInfo"].name;
  if (!role) return;
  let options = initRouter();
  let routesArr = calcRouters(options.routes, role);
  const rootRoute = routesArr.find((item) => item.path === "/");
  const menuRoutes = rootRoute && rootRoute.children;
  mergeI18nFromRoutes(i18n, menuRoutes);
  if (menuRoutes) {
    routesArr.forEach((v) => {
      router.addRoute(v);
    });
    store.commit("setting/setMenuData", menuRoutes);
  }
}
createMenus();
export { loginIgnore, initRouter, createMenus, router };
